<?python
  page_action = 'show'
?>
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:xi="http://www.w3.org/2001/XInclude"
      xmlns:py="http://genshi.edgewall.org/"><xi:include href="layout.html" />
  <head>
    <title>${page.title}</title>
  </head>
  <body>
    ${page.render()}
  </body>
</html>
