{% extends "layout.html" %}
{% set page_title = 'Missing Magic' %}
{% block page_body %}
  <p>
    The requested magic really does not exist. Maybe you want
    to look for it on the <a href="{{ h.url_for('static.index') }}">index</a>.
  </p>
{% endblock %}
