<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
  "http://www.w3.org/TR/html4/strict.dtd">
<html>
  <head>
    <title>Teeworlds Server Browser</title>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
    <link rel="stylesheet" type="text/css" href="${url_for('shared', file='style.css')}">
  </head>
  <body>
    <h1><a href="http://www.teeworlds.com/">Teeworlds</a> Server Browser</h1>
    <div class="contents">
      ${body}
    </div>
    <div class="footer">
      <a href="http://www.teeworls.com/">Teeworlds</a> Server Browser by
      <a href="http://lucumr.pocoo.org/">Armin Ronacher</a>, powered by
      <a href="http://werkzeug.pocoo.org/">Werkzeug</a>, licensed under
      the modified BSD license.
    </div>
  </body>
</html>
